@@ -33,6 +33,7 @@ const GdkPixbuf = imports.gi.GdkPixbuf;
 const Gtk = imports.gi.Gtk;
 const GtkClutter = imports.gi.GtkClutter;
 const Clutter = imports.gi.Clutter;
+const Pango = imports.gi.Pango;
 
 const Cairo = imports.cairo;
 const Tweener = imports.ui.tweener;
@@ -54,6 +55,15 @@ MainWindow.prototype = {
     _init : function(args) {
         args = args || {};
 
+        this._background = null;
+        this._pendingRenderer = null;
+        this._renderer = null;
+        this._texture = null;
+        this._toolbarActor = null;
+        this._fullScreenId = 0;
+        this._toolbarId = 0;
+        this._unFullScreenId = 0;
+
         this._mimeHandler = new MimeHandler.MimeHandler();
 
         this._application = args.application;
@@ -97,12 +107,7 @@ MainWindow.prototype = {
                                                   alpha: 255 }));
         this._mainGroup =  new Clutter.Group();
         this._stage.add_actor(this._mainGroup);
-
-        this._titleGroup =  new Clutter.Group();
-        this._stage.add_actor(this._titleGroup);
-
         this._mainGroup.set_opacity(0);
-        this._titleGroup.set_opacity(0);
     },
 
     _connectStageSignals : function() {
@@ -167,7 +172,8 @@ MainWindow.prototype = {
             key == Clutter.KEY_q)
             this._fadeOutWindow();
 
-        if (key == Clutter.KEY_f)
+        if (key == Clutter.KEY_f ||
+            key == Clutter.KEY_F11)
             this.toggleFullScreen();
     },
 
@@ -273,7 +279,7 @@ MainWindow.prototype = {
             if (this._renderer.clear)
                 this._renderer.clear();
 
-            delete this._renderer;
+            this._renderer = null;
         }
 
         /* create a temporary spinner renderer, that will timeout and show itself
@@ -306,7 +312,7 @@ MainWindow.prototype = {
         this._renderer.destroy();
 
         this._renderer = this._pendingRenderer;
-        delete this._pendingRenderer;
+        this._pendingRenderer = null;
 
         /* generate the texture and toolbar for the new renderer */
         this._createTexture();
@@ -316,7 +322,7 @@ MainWindow.prototype = {
     _createTexture : function() {
         if (this._texture) {
             this._texture.destroy();
-            delete this._texture;
+            this._texture = null;
         }
 
         this._texture = this._renderer.render();
@@ -341,11 +347,11 @@ MainWindow.prototype = {
      **************************************************************************/
     _onStageUnFullScreen : function() {
         this._stage.disconnect(this._unFullScreenId);
-        delete this._unFullScreenId;
+        this._unFullScreenId = 0;
 
 	/* We want the alpha background back now */
         this._background.destroy();
-        delete this._background;
+        this._background = null;
         this._createAlphaBackground();
 
         this._textureYAlign.factor = this._savedYFactor;
@@ -397,11 +403,11 @@ MainWindow.prototype = {
 
     _onStageFullScreen : function() {
         this._stage.disconnect(this._fullScreenId);
-        delete this._fullScreenId;
+        this._fullScreenId = 0;
 
         /* We want a solid black background */
         this._background.destroy();
-        delete this._background;
+        this._background = null;
 	this._createSolidBackground();
 
 	/* Fade in everything but the title */
@@ -476,8 +482,9 @@ MainWindow.prototype = {
      **************************************************************************/
     _createToolbar : function() {
         if (this._toolbarActor) {
+            this._removeToolbarTimeout();
             this._toolbarActor.destroy();
-            delete this._toolbarActor;
+            this._toolbarActor = null;
         }
 
         if (this._renderer.createToolbar)
@@ -507,14 +514,14 @@ MainWindow.prototype = {
     },
 
     _removeToolbarTimeout: function() {
-        Mainloop.source_remove(this._toolbarId);
-        delete this._toolbarId;
+        if (this._toolbarId != 0) {
+            Mainloop.source_remove(this._toolbarId);
+            this._toolbarId = 0;
+        }
     },
 
     _resetToolbar : function() {
-        if (this._toolbarId) {
-            this._removeToolbarTimeout();
-        } else {
+        if (this._toolbarId == 0) {
             Tweener.removeTweens(this._toolbarActor);
 
             this._toolbarActor.raise_top();
@@ -527,13 +534,14 @@ MainWindow.prototype = {
                              });
         }
 
+        this._removeToolbarTimeout();
         this._toolbarId = Mainloop.timeout_add(1500,
                                                Lang.bind(this,
                                                          this._onToolbarTimeout));
     },
 
     _onToolbarTimeout : function() {
-        delete this._toolbarId;
+        this._toolbarId = 0;
         Tweener.addTween(this._toolbarActor,
                          { opacity: 0,
                            time: 0.25,
@@ -547,24 +555,27 @@ MainWindow.prototype = {
      ************************ titlebar helpers ********************************
      **************************************************************************/
     _createTitle : function() {
-        if (this._titleLabel) {
-            this._titleActor.raise_top();
-            this._quitActor.raise_top();
+        if (this._titleGroup) {
+            this._titleGroup.raise_top();
             return;
         }
 
-        this._titleLabel = new Gtk.Label({ label: "" });
+        this._titleGroupLayout = new Clutter.BoxLayout();
+        this._titleGroup =  new Clutter.Box({ layout_manager: this._titleGroupLayout,
+                                              opacity: 0 });
+        this._stage.add_actor(this._titleGroup);
+
+        this._titleGroup.add_constraint(
+            new Clutter.BindConstraint({ source: this._stage,
+                                         coordinate: Clutter.BindCoordinate.WIDTH }));
+
+        this._titleLabel = new Gtk.Label({ label: "",
+					   ellipsize: Pango.EllipsizeMode.END,
+                                           margin: 6 });
         this._titleLabel.get_style_context().add_class("np-decoration");
         
         this._titleLabel.show();
         this._titleActor = new GtkClutter.Actor({ contents: this._titleLabel });
-        this._titleActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 0.5 }));
-        this._titleActor.add_constraint(
-            new Clutter.BindConstraint({ source: this._stage,
-                                         coordinate: Clutter.BindCoordinate.Y,
-                                         offset: 6 }));
 
         this._quitButton = 
             new Gtk.Button({ image: new Gtk.Image ({ "icon-size": Gtk.IconSize.MENU,
@@ -578,12 +589,17 @@ MainWindow.prototype = {
 
         this._quitActor = new GtkClutter.Actor({ contents: this._quitButton });
         this._quitActor.set_reactive(true);
-        this._quitActor.add_constraint(
-            new Clutter.AlignConstraint({ source: this._stage,
-                                          factor: 1.0 }));
 
-        this._titleGroup.add_actor(this._titleActor);
-        this._titleGroup.add_actor(this._quitActor);
+        let hidden = new Clutter.Actor();
+        let size = this._quitButton.get_preferred_size()[1];
+        hidden.set_size(size.width, size.height);
+
+        this._titleGroupLayout.pack(hidden, false, false, false,
+                                    Clutter.BoxAlignment.START, Clutter.BoxAlignment.START);
+        this._titleGroupLayout.pack(this._titleActor, true, true, false,
+                                    Clutter.BoxAlignment.CENTER, Clutter.BoxAlignment.START);
+        this._titleGroupLayout.pack(this._quitActor, false, false, false,
+                                    Clutter.BoxAlignment.END, Clutter.BoxAlignment.START);
     },
 
     /**************************************************************************
@@ -606,9 +622,7 @@ MainWindow.prototype = {
     },
 
     _fadeOutWindow : function() {
-        if (this._toolbarId) {
-            this._removeToolbarTimeout();
-        }
+        this._removeToolbarTimeout();
 
         Tweener.addTween(this._titleGroup,
                          { opacity: 0,
