@@ -37,7 +37,14 @@
 #include <gjs/gjs.h>
 
 #include <gtk/gtk.h>
+
+#ifdef GDK_WINDOWING_X11
+#include <X11/Xlib.h>
+#endif
+
 #include <clutter/clutter.h>
+#include <clutter/x11/clutter-x11.h>
+#include <clutter-gst/clutter-gst.h>
 #include <clutter-gtk/clutter-gtk.h>
 
 static void
@@ -141,6 +148,10 @@ main (int argc, char **argv)
   GtkSettings *settings;
   GError *error;
 
+#ifdef GDK_WINDOWING_X11
+  XInitThreads ();
+#endif
+
   clutter_x11_set_use_argb_visual (TRUE);
 
   if (gtk_clutter_init (&argc, &argv) < 0)
